from dateutil.parser import parse as date_parser
from mock import Mock
import pytest

from easytrakt.models import Show, Season, Episode


def test_search(client):
    client.request = Mock(return_value=[
        {"type": "show", "show": {"title": "foo"}}
    ])

    result = client.search("foo")
    assert result
    assert isinstance(result[0], Show)
    assert result[0].title == "foo"


def test_search_helpers(client):
    client.request = Mock(return_value=[
        {"type": "show", "show": {"title": "foo"}}
    ])

    result = client.shows("foo")
    client.request.assert_called_once_with(
        "search", {"query": "foo", "year": None, "type": "show"})
    assert result
    assert isinstance(result[0], Show)
    assert result[0].title == "foo"


def test_extend(client):
    client.request = Mock(return_value=[
        {"type": "show", "show": {"title": "foo", "ids": {"trakt": 1}}}
    ])

    show = client.search("foo")[0]
    assert show.title == "foo"
    assert show.ids.trakt == 1
    assert show.id == 1

    client.request = Mock(return_value={"overview": "this is the overview"})
    assert show.overview == "this is the overview"


def test_extend_fail(client):
    client.request = Mock(return_value=[
        {"type": "show", "show": {"title": "foo", "ids": {"trakt": 1}}}
    ])
    show = client.search("foo")[0]
    with pytest.raises(AttributeError):
        assert show.foo


def test_extend_seasons(client):
    client.request = Mock(return_value=[
        {"type": "show", "show": {"title": "foo", "ids": {"trakt": 1}}}
    ])

    show = client.search("foo")[0]
    assert show.title == "foo"
    client.request = Mock(return_value=[{"number": 0, "ids": {"trakt": 3}}])
    assert len(show.seasons) == 1
    assert isinstance(show.seasons[0], Season)
    assert show.seasons[0].number == 0
    season = show.seasons[0]
    client.request = Mock(return_value=[{"number": 1, "ids": {"trakt": 5}}])
    assert season.episodes
    assert isinstance(season.episodes[0], Episode)
    assert season.episodes[0].number == 1


def test_init_with_trakt_id(client):
    client.request = Mock(return_value={
        "title": "foo", "ids": {"trakt": 1}
    })
    show = Show(client, 1)
    assert show
    assert show.title == "foo"


def test_keys(client):
    assert Show(client, 1).keys() == [
        "aired_episodes", "airs", "available_translations", "certification",
        "country", "first_aired", "genres", "homepage", "ids", "images",
        "language", "network", "overview", "rating", "runtime", "seasons",
        "status", "title", "trailer", "updated_at", "votes", "year"]


def test_date_parser(client):
    client.request = Mock(return_value=[
        {"type": "show",
         "show": {
             "title": "foo",
             "ids": {"trakt": 1},
             "first_aired": None,
             "updated_at": "2006-10-08T04:00:00.000Z"
         }}
    ])

    show = client.search("foo")[0]
    assert show.first_aired is None
    assert show.updated_at == date_parser("2006-10-08T04:00:00.000Z")



import logging

import pytest

import easytrakt
from easytrakt import Client

easytrakt.logger.addHandler(logging.StreamHandler())
easytrakt.logger.setLevel(logging.DEBUG)


@pytest.yield_fixture
def client():
    yield Client()



from datetime import datetime

from dateutil.parser import parse as date_parser

from easytrakt.models import Show, Season, Episode, Movie


def test_search_show(client):
    result = client.search("Dexter")
    assert result
    show = result[0]
    assert isinstance(show, Show)
    assert show.title == "Dexter"
    assert show.ids.trakt == 1396
    assert show.id == 1396
    assert show.images.poster.full.startswith("http")


def test_search_movie(client):
    result = client.movies("The Big Lebowski")
    assert result
    movie = result[0]
    assert isinstance(movie, Movie)
    assert movie.title == "The Big Lebowski"
    assert movie.year == 1998
    assert movie.id == 84
    assert isinstance(movie.released, datetime)
    assert movie.released == date_parser("1998-03-06")
    assert movie.images.poster.full.startswith("http")
    assert movie.images.fanart.full.startswith("http")


def test_season(client):
    show = Show(client, 1396)
    seasons = show.seasons
    assert seasons
    season = seasons[1]
    assert isinstance(season, Season)
    assert season.number == 1
    assert season.trakt == 3999
    assert season.aired_episodes == 12
    assert season.images.poster.full.startswith("http")


def test_episodes(client):
    show = Show(client, 1396)
    episodes = show.seasons[1].episodes
    episode = episodes[1]
    assert isinstance(episode, Episode)
    assert episode.season == 1
    assert episode.number == 2
    assert episode.trakt == 74162
    assert episode.title == "Crocodile"
    assert isinstance(episode.first_aired, datetime)
    assert episode.first_aired == date_parser("2006-10-08T04:00:00.000Z")
    assert episode.images.screenshot.full.startswith("http")



from collections import defaultdict
from json import loads

from attrdict import AttrDict
from dateutil.parser import parse as date_parser


BASE_NESTED = {"ids": lambda c, d, p: AttrDict(d)}


def model_from_search_item(client, item):
    type_ = item["type"]

    return TYPE_MAP[type_](client, item[type_])


def images_generator(client, images, parent):
    out = AttrDict()
    for type_, sizes in images.items():
        for size, url in sizes.items():
            if type_ not in out:
                out[type_] = AttrDict()
            out[type_][size] = url
    return out


def date_generator(client, date, parent):
    if date:
        return date_parser(date)


class BaseModel(object):
    id_key = None
    extendables = defaultdict(list)
    nested = {}
    uri_section = ""
    heretical_identifier_structure = '{"ids": {"trakt": %s}}'

    def __init__(self, client, data, parent=None):
        self.client = client
        if isinstance(data, int):
            data = loads(self.heretical_identifier_structure % data)
        self._data = data
        self.nested.update(**BASE_NESTED)
        self.parent = parent

    def handle_nested(self, name, generator):
        if name != "ids":
            self.client.logger.debug(
                "%s building nested property '%s' with %s",
                self, name, self._data[name])
        return generator(self.client, self._data[name], self)

    def __getattr__(self, name):
        if name in self._data:
            if name in self.nested:
                return self.handle_nested(name, self.nested[name])
            return self._data[name]

        for extend_key, attributes in self.extendables.items():
            if name in attributes:
                self.client.logger.debug(
                    "%s is extending '%s' using '%s'",
                    self, name, extend_key)
                self.extend(name, **self.build_extend_kwargs(extend_key))
                if name in self._data:
                    break
        else:
            raise AttributeError(
                "{} has no (extendable) attribute: {}".format(self, name))
        return getattr(self, name)

    def build_extend_kwargs(self, extend_key):
        if extend_key.startswith("/"):
            return {"path": self.uri_path + extend_key}
        elif extend_key.startswith("-"):
            path_extension = None
            for extendable in self.parent.extendables:
                if extendable.startswith("/") and\
                        extendable[1:].startswith(self.type):
                    path_extension = extendable
            if path_extension is None:
                raise ValueError("no parent extension found for '%s'", self.type)
            return {
                "path": self.parent.uri_path + path_extension,
                "extend_flag": extend_key[1:]}

        return {"extend_flag": extend_key}

    def extend(self, name, extend_flag=None, path=None):
        uri = self.uri_path
        if path:
            uri = path

        if extend_flag == self.type:
            data = self.client.request(uri)
        elif extend_flag:
            data = self.client.request(uri, {"extended": extend_flag})
        else:
            data = self.client.request(uri)
        if isinstance(data, list):
            self._data[name] = data
            for entry in data:
                base = BaseModel(self.client, entry)
                if base.trakt == self.trakt:
                    self.client.logger.debug(
                        "%s is applying '%s' with entry %s", self, name, base)
                    self._data.update(**entry)
                    break
        else:
            self.client.logger.debug(
                "%s is updating its data, because of '%s' with %s",
                self, name, data)
            self._data.update(**data)
        return data

    def keys(self):
        keys = []
        for attributes in self.extendables.values():
            keys.extend(attributes)
        return sorted(keys)

    @property
    def type(self):
        return self.__class__.__name__.lower()

    @property
    def uri_path(self):
        if self.parent is None:
            return self.uri_section.format(self=self)
        return self.parent.uri_path + self.uri_section.format(self=self)

    @property
    def id(self):
        return getattr(self, self.id_key)

    @property
    def trakt(self):
        return self.ids.trakt

    def __repr__(self):
        return "<{}>".format(self.__class__.__name__)

    __str__ = __repr__


class Movie(BaseModel):
    id_key = "trakt"
    extendables = {
        "movie": ["title", "ids", "year"],
        "images": ["images"],
        "full": ["overview", "released", "tagline", "trailer", "rating",
                 "votes", "language", "available_translations", "genres",
                 "certification"],
        "/seasons": ["seasons"]}
    nested = {
        "released": date_generator,
        "images": images_generator
    }
    uri_section = "/movies/{self.id}"


class Episode(BaseModel):
    id_key = "number"
    extendables = {
        "images": ["images"],
        "full": [
            "number_abs", "overview", "first_aired", "updated_at", "rating",
            "updated_at", "rating", "votes", "available_translations"],
        "episodes": ["episodes"]}
    nested = {
        "first_aired": date_generator,
        "images": images_generator
    }
    uri_section = "/episodes/{self.number}"


class Season(BaseModel):
    id_key = "number"
    extendables = {
        "season": ["ids"],
        "-images": ["images"],
        "-full": [
            "rating", "votes", "episode_count", "aired_episodes", "overview"],
        "episodes": ["episodes"]}
    nested = {
        "episodes": lambda c, ee, p: [Episode(c, s, p) for s in ee],
        "images": images_generator
    }
    uri_section = "/seasons/{self.number}"


class Show(BaseModel):
    id_key = "trakt"
    extendables = {
        "show": ["title", "ids"],
        "images": ["images"],
        "full": [
            "status", "rating", "votes", "network", "language",
            "overview", "first_aired", "aired_episodes", "updated_at",
            "country", "available_translations", "airs", "year",
            "certification", "runtime", "homepage", "trailer", "genres"],
        "/seasons": ["seasons"]}
    nested = {
        "seasons": lambda c, ss, p: [Season(c, s, p) for s in ss],
        "airs": lambda c, d, p: AttrDict(d),
        "images": images_generator,
        "updated_at": date_generator,
        "first_aired": date_generator
    }
    uri_section = "shows/{self.ids.trakt}"


class Person(BaseModel):
    pass


class List(BaseModel):
    pass


TYPE_MAP = {
    "movie": Movie,
    "show": Show,
    "person": Person,
    "episode": Episode,
    "list": List
}



__author__ = 'dennis.lutter'

from functools import partial
import logging

from cachecontrol import CacheControl
import requests

from models import model_from_search_item
from models import TYPE_MAP

BASE_URL = "https://api-v2launch.trakt.tv"
logger = logging.getLogger("easytrakt")


class Client(object):

    def __init__(self, session=None, verify_ssl=True, url=BASE_URL):
        session = session or requests.session()
        session.headers = {
            "Content-type": "application/json",
            "trakt-api-key": getattr(session, "client_id", ""),
            "trakt-api-version": 2
        }
        self.session = CacheControl(session)

        self.logger = logger
        self.verify_ssl = verify_ssl
        self.base_url = url

        for type_ in TYPE_MAP:
            setattr(self, type_ + "s", partial(self.search, type=type_))

    def search(self, query, type=None, year=None):
        params = {
            "query": query,
            "type": type,
            "year": year
        }
        return [model_from_search_item(self, item)
                for item in self.request("search", params)]

    def request(self, uri_path, params=None):
        self.logger.info("call to: %s with %s", uri_path, params)
        result = self.session.get(
            "{}/{}".format(self.base_url, uri_path),
            params=params,
            verify=self.verify_ssl
        )
        result.raise_for_status()
        return result.json()



import logging
import os

import pytest
from requests_oauthlib import OAuth2Session

import easytrakt
from easytrakt import Client

easytrakt.logger.addHandler(logging.StreamHandler())
easytrakt.logger.setLevel(logging.DEBUG)

CLIENT_ID = "8224c88cc14f27ab6e0d894dec500f7c46bd5de71fdc1b27cac6be8c027f023b"


@pytest.yield_fixture
def client():
    token = {
        "access_token": os.environ["TRAKT_TOKEN"],
        "created_at": 1433447370,
        "expires_in": 7776000,
        "expires_at": 1441223370.73398,
        "token_type": "bearer",
        "scope": ["public"],
        "refresh_token": ""
    }
    session = OAuth2Session(
        CLIENT_ID,
        token=token
    )
    yield Client(session)
